//uart.c

#include "uart.h"


/*-----Az USART UCSR0A allapotanak lekerdezese-----*/
char isCharAvailable(void) {
  //Az USART Allapot es Vezerlo Regiszter RXC0 bitje
  //vajon azt jelzi, hogy karakter erkezett?
  if (UCSRA & (1 << RXC)) return(1);
  else return(0);
}



/*-----Karakterfogado fuggveny-----*/
char receiveChar(void) {
  //Az UDR regiszterben levo karakter visszaadasa
  return UDR;
}



/*-----Karakterkuldo fuggveny-----*/
void sendChar(char data) {
  int i = 0;

  //Az adat elkuldesehez USART segitsegevel betesszuk azt az adatregiszterbe
  UDR = data;
  
  //Megvizsgaljuk engedelyezettek-e a globalis megszakitasok
  if (SREG & (1 << SREG_I)) {
    //Varni mig el lesz kuldve egy byte vagy szamlalunk felfele
    while ((!(UCSRA & (1 << TXC))) && (i < 32000)) {
      i++;
    }
  }
  else //Varni mig el lesz kuldve egy byte
    while (!(UCSRA & (1 << TXC)));

  //A TXC flag torlese
  UCSRA |= (1 << TXC);
}



/*-----Karakterlanc-kuldo fuggveny-----*/
void sendString(char *s) {
  int i = 0;
  
  while (i < 64) { //Ha rossz a sztring nem foglalkozunk vele
    if (s[i] == '\0') break; //Kilepes, ha sztring vege
    sendChar(s[i++]);
  }
}



/*-----USART inicializalo fuggveny-----*/
void USARTinit(void) {
//Az UART baudrate regiszter tartalmanak kiszamitasa  
  UBRRH = (uint8_t) (UART_BAUD_CALC(UART_BAUD_RATE,F_CPU) >> 8);
  UBRRL = (uint8_t) UART_BAUD_CALC(UART_BAUD_RATE, F_CPU);

//Az ado es a vevo engedelyezese
  UCSRB = (1 << RXEN) | (1 << TXEN) | (0 << RXCIE) | (0 << UDRIE);

  //Az USART beallitasa 8 bites, paritasnelkuli, 1 stop bit aszinkron adasra-vetelre
  UCSRC = (3 << UCSZ0) | (0 << UPM0) | (0 << USBS) | (0 << UMSEL) | (0 << UCPOL) ;
  //UCPOL bit = 0 aszinkron atvitelnel
  
  //Globalis megszakitasok engedelyezese
  sei();
}
